package com.dky.calculate;


import com.dky.utils.ConfigReader;
import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.enums.DeviceSubType;
import com.dky.utils.enums.MetalKinTemps;
import com.dky.utils.result.MatchedDevice;

import java.util.*;
import java.util.stream.Collectors;

public class MaterialKilnScheme {


    public static List<MatchedDevice> calSchemeByTechType(Double workArea, List<SysDeviceHeatScene> sysDeviceHeatSceneList) {
        // 对技术类型下的设备list按照单台设备工作容积进行排序
        sysDeviceHeatSceneList.sort((o1, o2) -> Double.compare(o2.getDevWorkArea(), o1.getDevWorkArea()));
        Double remainArea = workArea;
        // 遍历设备，根据工作容积进行匹配设备数量
        List<MatchedDevice> matchedDeviceList = new ArrayList<>();

        for (int i = 0; i < sysDeviceHeatSceneList.size(); i++) {
            SysDeviceHeatScene deviceHeatScene = sysDeviceHeatSceneList.get(i);
            double v = remainArea / deviceHeatScene.getDevWorkArea();
            int count = (int) (v);
            if (v < 1) {
                count = count + 1;
            }
            remainArea = remainArea - count * deviceHeatScene.getDevWorkArea();
            matchedDeviceList.add(new MatchedDevice(count, deviceHeatScene));
            if (remainArea <= 0) {
                // 当总供暖面积已经满足需求时，跳出循环，不再选择其他设备
                break;
            } else {
                if (i == sysDeviceHeatSceneList.size() - 1 && remainArea > 0) {
                    MatchedDevice device = matchedDeviceList.get(matchedDeviceList.size() - 1);
                    device.setCount(device.getCount() + 1);
                }
            }
        }
        return matchedDeviceList;
    }

    public static List<SysDeviceHeatScene> getSchemeList(String userIndustry, Double needTemp, List<SysDeviceHeatScene> sysDeviceHeatSceneList) {
        // 建筑行业编码
        String[] property1 = ConfigReader.getPropertyByFile("industry_codes1", "IndustryCode.properties").split(",");
        // 黑色金属矿采选业 行业编码
        String[] property2 = ConfigReader.getPropertyByFile("industry_codes2", "IndustryCode.properties").split(",");
        Map<String, Object> map = new HashMap<>();
        List<SysDeviceHeatScene> sceneDevicesList = new ArrayList<>();

        // 将sysDeviceHeatSceneList流转换为排序后的ArrayList<SysDeviceHeatScene>对象。
        // 排序依据是SysDeviceHeatScene对象的getDevTechType方法返回的值。
        ArrayList<SysDeviceHeatScene> ary = sysDeviceHeatSceneList.stream().collect(
                Collectors.collectingAndThen(
                        // 使用TreeSet收集元素，以保证收集后的集合是按照getDevTechType方法返回值排序的。
                        Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(SysDeviceHeatScene::getDevTechType))),
                        // 将TreeSet转换为ArrayList。
                        ArrayList::new
                )
        );

        // 遍历ary数组中的每个元素
        ary.forEach((item) -> {
            List<SysDeviceHeatScene> list = new ArrayList<>();
            // 遍历sysDeviceHeatSceneList列表，筛选出与当前item的设备技术类型相同的设备
            sysDeviceHeatSceneList.forEach((device) -> {
                if (device.getDevTechType().equals(item.getDevTechType())) {
                    list.add(device); // 将符合条件的设备添加到list中
                }
            });
            // 将按照设备技术类型分组后的设备列表，添加到map中
            map.put(item.getDevTechType(), list);
        });

        // 判断用户行业是否属于建筑
        if (new HashSet<>(Arrays.asList(property1)).contains(userIndustry)){
            sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Material_Kiln.getDesc());
        } else {
            // 判断用户行业是否属于黑色金属矿采选业
            if (new HashSet<>(Arrays.asList(property2)).contains(userIndustry)) {
                sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Electric_Arc.getDesc());
            } else {
                if (needTemp >= MetalKinTemps.Metal_Kiln_Direct_Heat_Min_Temp.getTemp() && needTemp <= MetalKinTemps.Metal_Kiln_Direct_Heat_Max_Temp.getTemp()){
                    sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Direct_Heat.getDesc());
                } else if (needTemp >= MetalKinTemps.Metal_Kiln_Interstitial_Heat_Min_Temp.getTemp() && needTemp <= MetalKinTemps.Metal_Kiln_Interstitial_Heat_Max_Temp.getTemp()){
                    sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Interstitial_Heat.getDesc());
                } else if (needTemp >= MetalKinTemps.Metal_Kiln_Induction_Min_Temp.getTemp() && needTemp <= MetalKinTemps.Metal_Kiln_Induction_Max_Temp.getTemp()) {
                    sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Induction.getDesc());
                }
            }
        }

        return sceneDevicesList;

    }

    public static List<SysDeviceHeatScene> createDevList() {
        List<SysDeviceHeatScene> deviceList = new ArrayList<>();

        // 模拟从数据库中检索数据
        Object[][] data =
                {
                        {1, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电阻类）", 0.95, 360.00, 80000.00, 60000.00, 180000.00, 3000.00, 15, 4500, null, null,null,null,null,null},
                        {2, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电阻类）", 0.95, 720.00, 100000.00, 60000.00, 180000.00, 6000.00, 15, 4500, null, null,null,null,null,null},
                        {3, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电阻类）", 0.95, 1440.00, 160000.00, 60000.00, 180000.00, 12000.00, 15, 4500, null, null,null,null,null,null},
                        {4, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电极类）", 0.97, 360.00, 280000.00, 60000.00, 180000.00, 3000.00, 15, 4500, null, null,null,null,null,null},
                        {5, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电极类）", 0.97, 720.00, 460000.00, 60000.00, 180000.00, 6000.00, 15, 4500, null, null,null,null,null,null},
                        {6, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电极类）", 0.97, 1440.00, 900000.00, 60000.00, 180000.00, 12000.00, 15, 4500, null, null,null,null,null,null},
                        {7, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电磁涡流类）", 0.98, 360.00, 150000.00, 60000.00, 180000.00, 3000.00, 15, 4500, null, null,null,null,null,null},
                        {8, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电磁涡流类）", 0.98, 720.00, 180000.00, 60000.00, 180000.00, 6000.00, 15, 4500, null, null,null,null,null,null},
                        {9, "电锅炉", "工业供热电锅炉", "直热式电锅炉（电磁涡流类）", 0.98, 1440.00, 300000.00, 60000.00, 180000.00, 12000.00, 15, 4500, null, null,null,null,null,null},
                        {10, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（固体蓄热）", 0.95, 360.00, 360000.00, 60000.00, 180000.00, 3000.00, 15, 4500, null, null,null,null,null,null},
                        {11, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（固体蓄热）", 0.95, 720.00, 720000.00, 60000.00, 180000.00, 6000.00, 15, 4500, null, null,null,null,null,null},
                        {12, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（固体蓄热）", 0.95, 1440.00, 1440000.00, 60000.00, 180000.00, 12000.00, 15, 4500, null, null,null,null,null,null},
                        {13, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（相变蓄热）", 0.98, 360.00, 120000.00, 60000.00, 180000.00, 3000.00, 15, 4500, null, null,null,null,null,null},
                        {14, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（相变蓄热）", 0.98, 720.00, 160000.00, 60000.00, 180000.00, 6000.00, 15, 4500, null, null,null,null,null,null},
                        {15, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（相变蓄热）", 0.98, 1440.00, 280000.00, 60000.00, 180000.00, 12000.00, 15, 4500, null, null,null,null,null,null},
                        {16, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（水蓄）", 0.97, 360.00, 110000.00, 60000.00, 180000.00, 3000.00, 15, 4500, null, null,null,null,null,null},
                        {17, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（水蓄）", 0.97, 720.00, 150000.00, 60000.00, 180000.00, 6000.00, 15, 4500, null, null,null,null,null,null},
                        {18, "电锅炉", "工业供热电锅炉", "蓄热式电锅炉（水蓄）", 0.97, 1440.00, 260000.00, 60000.00, 180000.00, 12000.00, 15, 4500, null, null,null,null,null,null},
                        {19, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电阻类）", 0.95, 360.00, 80000.00, 60000.00, 180000.00, 3000.00, 15, null, null, null,null,null,null,null},
                        {20, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电阻类）", 0.95, 720.00, 100000.00, 60000.00, 180000.00, 6000.00, 15, null, null, null,null,null,null,null},
                        {21, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电阻类）", 0.95, 1440.00, 160000.00, 60000.00, 180000.00, 12000.00, 15, null, null, null,null,null,null,null},
                        {22, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电极类）", 0.97, 360.00, 280000.00, 60000.00, 180000.00, 3000.00, 15, null, null, null,null,null,null,null},
                        {23, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电极类）", 0.97, 720.00, 460000.00, 60000.00, 180000.00, 6000.00, 15, null, null, null,null,null,null,null},
                        {24, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电极类）", 0.97, 1440.00, 900000.00, 60000.00, 180000.00, 12000.00, 15, null, null, null,null,null,null,null},
                        {25, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电磁涡流类）", 0.98, 360.00, 150000.00, 60000.00, 180000.00, 3000.00, 15, null, null, null,null,null,null,null},
                        {26, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电磁涡流类）", 0.98, 720.00, 180000.00, 60000.00, 180000.00, 6000.00, 15, null, null, null,null,null,null,null},
                        {27, "电锅炉", "供冷/暖电锅炉", "直热式电锅炉（电磁涡流类）", 0.98, 1440.00, 300000.00, 60000.00, 180000.00, 12000.00, 15, null, null, null,null,null,null,null},
                        {28, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（固体蓄热）", 0.95, 360.00, 360000.00, 60000.00, 180000.00, 3000.00, 15, null, null, null,null,null,null,null},
                        {29, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（固体蓄热）", 0.95, 720.00, 720000.00, 60000.00, 180000.00, 6000.00, 15, null, null, null,null,null,null,null},
                        {30, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（固体蓄热）", 0.95, 1440.00, 1440000.00, 60000.00, 180000.00, 12000.00, 15, null, null, null,null,null,null,null},
                        {31, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（相变蓄热）", 0.98, 360.00, 120000.00, 60000.00, 180000.00, 3000.00, 15, null, null, null,null,null,null,null},
                        {32, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（相变蓄热）", 0.98, 720.00, 160000.00, 60000.00, 180000.00, 6000.00, 15, null, null, null,null,null,null,null},
                        {33, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（相变蓄热）", 0.98, 1440.00, 280000.00, 60000.00, 180000.00, 12000.00, 15, null, null, null,null,null,null,null},
                        {34, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（水蓄）", 0.97, 360.00, 110000.00, 60000.00, 180000.00, 3000.00, 15, null, null, null,null,null,null,null},
                        {35, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（水蓄）", 0.97, 720.00, 150000.00, 60000.00, 180000.00, 6000.00, 15, null, null, null,null,null,null,null},
                        {36, "电锅炉", "供冷/暖电锅炉", "蓄热式电锅炉（水蓄）", 0.97, 1440.00, 260000.00, 60000.00, 180000.00, 12000.00, 15, null, null, null,null,null,null,null},
                        {37, "热泵", "供冷/暖", "水源热泵", 4.00, 25.00, 45000.00, 60000.00, 180000.00, 500.00, 15, null, null, null,null,null,null,null},
                        {38, "热泵", "供冷/暖", "水源热泵", 4.00, 45.00, 75000.00, 60000.00, 180000.00, 1000.00, 15, null, null, null,null,null,null,null},
                        {39, "热泵", "供冷/暖", "水源热泵", 4.00, 90.00, 140000.00, 60000.00, 180000.00, 2000.00, 15, null, null, null,null,null,null,null},
                        {40, "热泵", "供冷/暖", "土壤源热泵", 4.00, 25.00, 45000.00, 60000.00, 180000.00, 500.00, 15, null, null, null,null,null,null,null},
                        {41, "热泵", "供冷/暖", "土壤源热泵", 4.00, 45.00, 75000.00, 60000.00, 180000.00, 1000.00, 15, null, null, null,null,null,null,null},
                        {42, "热泵", "供冷/暖", "土壤源热泵", 4.00, 90.00, 140000.00, 60000.00, 180000.00, 2000.00, 15, null, null, null,null,null,null,null},
                        {43, "热泵", "供冷/暖", "空气源热泵", 4.00, 25.00, 45000.00, 60000.00, 180000.00, 500.00, 15, null, null, null,null,null,null,null},
                        {44, "热泵", "供冷/暖", "空气源热泵", 4.00, 45.00, 75000.00, 60000.00, 180000.00, 1000.00, 15, null, null, null,null,null,null,null},
                        {45, "热泵", "供冷/暖", "空气源热泵", 4.00, 90.00, 140000.00, 60000.00, 180000.00, 2000.00, 15, null, null, null,null,null,null,null},
                        {46, "热泵", "供冷/暖", "低品位余热源（如电厂低温循环水）热泵", 4.00, 25.00, 45000.00, 60000.00, 180000.00, 500.00, 15, null, null, null,null,null,null,null},
                        {47, "热泵", "供冷/暖", "低品位余热源（如电厂低温循环水）热泵", 4.00, 45.00, 75000.00, 60000.00, 180000.00, 1000.00, 15, null, null, null,null,null,null,null},
                        {48, "热泵", "供冷/暖", "低品位余热源（如电厂低温循环水）热泵", 4.00, 90.00, 140000.00, 60000.00, 180000.00, 2000.00, 15, null, null, null,null,null,null,null},
                        {49, "热泵", "供冷/暖", "高温蒸汽热泵", 3.00, 45.00, 380000.00, 60000.00, 180000.00, 1000.00, 15, null, null, null,null,null,null,null},
                        {50, "热泵", "供冷/暖", "高温蒸汽热泵", 3.00, 120.00, 550000.00, 60000.00, 180000.00, 2000.00, 15, 1250, null, null,null,null,null,null},
                        {51, "电厨炊", "电厨炊", "电磁灶", null, 12, 3500.00, 60000.00, 180000.00, null, 15, 1250, 1, 50,null,null,null,null},
                        {52, "电厨炊", "电厨炊", "蒸锅", null, 9, 2200.00, 60000.00, 180000.00, null, 15, 1250, 1, 50,null,null,null,null},
                        {53, "电厨炊", "电厨炊", "汤炉", null, 6, 1500.00, 60000.00, 180000.00, null, 15, 1250, 1, 50,null,null,null,null},
                        {54, "电厨炊", "电厨炊", "煮炉", null, 8, 2100.00, 60000.00, 180000.00, null, 15, 1250, 1, 50,null,null,null,null},
                        {55, "电厨炊", "电厨炊", "饼铛", null, 4, 1600.00, 60000.00, 180000.00, null, 15, 1250, 1, 50,null,null,null,null},
                        {56, "电厨炊", "电厨炊", "电磁灶", null, 20, 4500.00, 60000.00, 180000.00, null, 15, 1250, 51, 100,null,null,null,null},
                        {57, "电厨炊", "电厨炊", "蒸锅", null, 12, 2500.00, 60000.00, 180000.00, null, 15, 1250, 51, 100,null,null,null,null},
                        {58, "电厨炊", "电厨炊", "汤炉", null, 9, 2500.00, 60000.00, 180000.00, null, 15, 1250, 51, 100,null,null,null,null},
                        {59, "电厨炊", "电厨炊", "煮炉", null, 12, 2900.00, 60000.00, 180000.00, null, 15, 1250, 51, 100,null,null,null,null},
                        {60, "电厨炊", "电厨炊", "饼铛", null, 5, 1700.00, 60000.00, 180000.00, null, 15, 1250, 51, 100,null,null,null,null},
                        {61, "电厨炊", "电厨炊", "电磁灶", null, 30, 5900.00, 60000.00, 180000.00, null, 15, 1250, 101, null,null,null,null,null},
                        {62, "电厨炊", "电厨炊", "蒸锅", null, 12, 3900.00, 60000.00, 180000.00, null, 15, 1250, 101, null,null,null,null,null},
                        {63, "电厨炊", "电厨炊", "汤炉", null, 12, 2900.00, 60000.00, 180000.00, null, 15, 1250, 101, null,null,null,null,null},
                        {64, "电厨炊", "电厨炊", "煮炉", null, 18, 3200.00, 60000.00, 180000.00, null, 15, 1250, 101, null,null,null,null,null},
                        {65, "电厨炊", "电厨炊", "饼铛", null, 5, 1700.00, 60000.00, 180000.00, null, 15, 1250, 101, null,null,null,null,null},
                        {66, "电窑炉", "建材电窑炉", "建材电窑炉", null, 25, 26000.00, 60000.00, 180000.00, null, 15, 4500, null, null,0.20,2500.00,500.00,4.00},
                        {67, "电窑炉", "建材电窑炉", "建材电窑炉", null, 145, 140000.00, 60000.00, 180000.00, null, 15, 4500, null, null,1.50,2500.00,3750.00,4.00},
                        {68, "电窑炉", "建材电窑炉", "建材电窑炉", null, 300, 250000.00, 60000.00, 180000.00, null, 15, 4500, null, null,3.00,2500.00,7500.00,4.00},
                        {69, "电窑炉", "冶金窑炉", "直接加热电阻炉", null, 25, 26000.00, 60000.00, 180000.00, null, 15, 4500, null, null,0.20,7800.00,1560.00,4.00},
                        {70, "电窑炉", "冶金窑炉", "直接加热电阻炉", null, 145, 140000.00, 60000.00, 180000.00, null, 15, 4500, null, null,1.50,7800.00,11700.00,4.00},
                        {71, "电窑炉", "冶金窑炉", "直接加热电阻炉", null, 300, 250000.00, 60000.00, 180000.00, null, 15, 4500, null, null,3.00,7800.00,23400.00,4.00},
                        {72, "电窑炉", "冶金窑炉", "间接加热电阻炉（钟罩式光亮退火炉)", null, 25, 60000.00, 60000.00, 180000.00, null, 15, 4500, null, null,0.20,7800.00,1560.00,4.00},
                        {73, "电窑炉", "冶金窑炉", "间接加热电阻炉（钟罩式光亮退火炉)", null, 145, 250000.00, 60000.00, 180000.00, null, 15, 4500, null, null,1.50,7800.00,11700.00,4.00},
                        {74, "电窑炉", "冶金窑炉", "间接加热电阻炉（钟罩式光亮退火炉)", null, 300, 450000.00, 60000.00, 180000.00, null, 15, 4500, null, null,3.00,7800.00,23400.00,4.00},
                        {75, "电窑炉", "冶金窑炉", "电弧炉（钢包精炼LF炉）", null, 25, 28000.00, 60000.00, 180000.00, null, 15, 4500, null, null,0.20,7800.00,1560.00,4.00},
                        {76, "电窑炉", "冶金窑炉", "电弧炉（钢包精炼LF炉）", null, 145, 150000.00, 60000.00, 180000.00, null, 15, 4500, null, null,1.50,7800.00,11700.00,4.00},
                        {77, "电窑炉", "冶金窑炉", "电弧炉（钢包精炼LF炉）", null, 300, 260000.00, 60000.00, 180000.00, null, 15, 4500, null, null,3.00,7800.00,23400.00,4.00},
                        {78, "电窑炉", "冶金窑炉", "中/高频感应电炉", null, 25, 40000.00, 60000.00, 180000.00, null, 15, 4500, null, null,0.20,7800.00,1560.00,4.00},
                        {79, "电窑炉", "冶金窑炉", "中/高频感应电炉", null, 145, 190000.00, 60000.00, 180000.00, null, 15, 4500, null, null,1.50,7800.00,11700.00,4.00},
                        {80, "电窑炉", "冶金窑炉", "中/高频感应电炉", null, 300, 270000.00, 60000.00, 180000.00, null, 15, 4500, null, null,3.00,7800.00,23400.00,4.00},
                };

        // 创建对象并添加到列表
        for (Object[] row : data) {
            int id = (int) row[0];
            String devType = (String) row[1];
            String devSubType = (String) row[2];
            String devTechType = (String) row[3];
            double devPrice = (double) row[6];
            double devSubstituteLaborCost = (double) row[7];
            double laborCost = (double) row[8];


            SysDeviceHeatScene device = new SysDeviceHeatScene(
                    id, devType, devSubType, devTechType, isHave2(row[4]), isHave2(row[5]), devPrice,
                    devSubstituteLaborCost, laborCost, isHave2(row[9]), isHave(row[10]), isHave(row[11]),
                    isHave(row[12]), isHave(row[13]),
                    isHave2(row[14]), isHave2(row[15]),
                    isHave2(row[16]), isHave2(row[17]),null);

            deviceList.add(device);
        }
        return deviceList;
    }

    public static Integer isHave(Object args) {
        if (null != args) {
            return Integer.parseInt(args.toString());
        } else {
            return null;
        }
    }

    public static Double isHave2(Object args) {
        if (null != args) {
            return Double.parseDouble(args.toString());
        } else {
            return null;
        }
    }


}
