package com.dky.stirtpar;


import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.dky.modelI.DntdModelI;
import com.dky.utils.CalculateUtils;
import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.entity.SysModelParam;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class StirparModelCalculate {

    static DecimalFormat df = new DecimalFormat("#.00");

    /**
     * 计算热源消耗总量
     * @param list 参数列表
     * @param peopleNum 人口
     * @param averageGdp 人均GDP
     * @param U 城市化率
     * @param I 工业增加值
     * @param II 工业能源强度
     * @return
     */
    public static Double energyConsumeStirCoefficient(List<SysModelParam> list, Integer peopleNum, Double averageGdp, Double U, Double I, Double II) {
        List<Map> mapList = getThisModel(0, list);
        Double a = 1.0;
        Double b1 = 1.0;
        Double b2 = 1.0;
        Double b3 = 1.0;
        Double b4 = 1.0;
        Double b5 = 1.0;
        Double b6 = 1.0;
        Double e = 1.0;

        for (Map param : mapList){
            if ("a".equals(param.get("key"))) {
                a = (Double) param.get("value");
            } else if ("b1".equals(param.get("key"))) {
                b1 = (Double) param.get("value");
            } else if ("b2".equals(param.get("key"))) {
                b2 = (Double) param.get("value");
            } else if ("b3".equals(param.get("key"))) {
                b3 = (Double) param.get("value");
            } else if ("b4".equals(param.get("key"))) {
                b4 = (Double) param.get("value");
            } else if ("b55".equals(param.get("key"))) {
                b5 = (Double) param.get("value");
            } else if ("b6".equals(param.get("key"))) {
                b6 = (Double) param.get("value");
            } else if ("e".equals(param.get("key"))) {
                e = (Double) param.get("value");
            }
        }

        Double add1 = CalculateUtils.add(a, b1 * Math.log(peopleNum));
        Double add2 = CalculateUtils.add(add1, b2 * Math.log(U));
        Double add3 = CalculateUtils.add(add2, b3 * Math.log(averageGdp));
        Double add4 = CalculateUtils.add(add3, b4 * Math.pow(Math.log(averageGdp), 2));
        Double add5 = CalculateUtils.add(add4, b5 * Math.log(I));
        Double add6 = CalculateUtils.add(add5, b6 * Math.log(II));
        Double add7 = CalculateUtils.add(add6, e);

        return Math.exp(add7);
    }

    /**
     * 计算电耗消耗总量
     * @param list 参数列表
     * @param CT 能源消耗量
     * @param peopleNum 人口
     * @param averageGdp 人均GDP
     * @param U 城市化率
     * @param PR 电力价格
     * @param T 年均温度
     * @return
     */
    public static Double electricConsumeStirCoefficient(List<SysModelParam> list, Double CT, Integer peopleNum, Double averageGdp, Double U, Double PR, Double T) {
        List<Map> mapList = getThisModel(1, list);
        Double a = 1.0;
        Double b1 = 1.0;
        Double b2 = 1.0;
        Double b3 = 1.0;
        Double b4 = 1.0;
        Double b5 = 1.0;
        Double b6 = 1.0;
        Double b7 = 1.0;
        Double e = 1.0;

        for (Map param : mapList){
            if ("a".equals(param.get("key"))) {
                a = (Double) param.get("value");
            } else if ("b1".equals(param.get("key"))) {
                b1 = (Double) param.get("value");
            } else if ("b2".equals(param.get("key"))) {
                b2 = (Double) param.get("value");
            } else if ("b3".equals(param.get("key"))) {
                b3 = (Double) param.get("value");
            } else if ("b4".equals(param.get("key"))) {
                b4 = (Double) param.get("value");
            } else if ("b55".equals(param.get("key"))) {
                b5 = (Double) param.get("value");
            } else if ("b6".equals(param.get("key"))) {
                b6 = (Double) param.get("value");
            } else if ("b7".equals(param.get("key"))) {
                b7 = (Double) param.get("value");
            } else if ("e".equals(param.get("key"))) {
                e = (Double) param.get("value");
            }
        }

        Double add1 = CalculateUtils.add(a, b1 * Math.log(CT));
        Double add2 = CalculateUtils.add(add1, b2 * Math.log(peopleNum));
        Double add3 = CalculateUtils.add(add2, b3 * Math.log(U));
        Double add4 = CalculateUtils.add(add3, b4 * Math.log(U));
        Double add5 = CalculateUtils.add(add4, b5 * Math.log(averageGdp));
        Double add6 = CalculateUtils.add(add5, b6 * Math.log(PR));
        Double add7 = CalculateUtils.add(add6, b7 * Math.log(T));
        Double add8 = CalculateUtils.add(add7, e);

        return Math.exp(add8);
    }



    public static JSONObject toJSON(String fileName) {
        // 使用ClassLoader的getResourceAsStream方法读取资源文件
        InputStream inputStream = StirparModelCalculate.class.getClassLoader().getResourceAsStream(fileName);
        if (inputStream == null) {
            System.out.println("读取文件失败");
        }
        // 使用BufferedReader读取InputStream中的内容
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8));
        StringBuilder stringBuilder = new StringBuilder();
        String currentLine;
        try {
            while ((currentLine = reader.readLine()) != null) {
                stringBuilder.append(currentLine);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        // 关闭BufferedReader
        try {
            reader.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return new JSONObject(stringBuilder.toString());
    }



    public static JSONObject createReport(JSONObject jsonObject) {
        JSONObject electricInfo = new JSONObject();
        JSONObject energyInfo = new JSONObject();

        try {
            energyInfo = (JSONObject) jsonObject.get("energyInfo");
            electricInfo = (JSONObject) jsonObject.get("electricInfo");
        } catch (Exception e) {
            e.printStackTrace();
        }

        List<SysModelParam> sysParamList = new ArrayList<>();
        JSONArray specArray = jsonObject.getJSONArray("sysModelParam");
        for (int i = 0; i < specArray.size(); i++) {
            SysModelParam devSpec = specArray.getJSONObject(i).toBean(SysModelParam.class);
            sysParamList.add(devSpec);
        }
        Integer peopleNum = Integer.parseInt(new DecimalFormat("#").format(energyInfo.get("peopleNum")));
        Double averageGdp = Double.parseDouble(energyInfo.get("averageGdp").toString());
        Double U = Double.parseDouble(energyInfo.get("ut").toString());
        Integer peopleNum2 = Integer.parseInt(new DecimalFormat("#").format(electricInfo.get("peopleNum")));
        Double averageGdp2 = Double.parseDouble(electricInfo.get("averageGdp").toString());
        Double U2 = Double.parseDouble(electricInfo.get("ut").toString());
        Double I = Double.parseDouble(energyInfo.get("i1").toString());
        Double II = Double.parseDouble(energyInfo.get("i2").toString());
        Double PR = Double.parseDouble(electricInfo.get("PR").toString());
        Double CT = Double.parseDouble(electricInfo.get("CT").toString());
        Double T = Double.parseDouble(electricInfo.get("T").toString());

        JSONObject resultJsonObject = new JSONObject();
         resultJsonObject.put("CT", df.format(energyConsumeStirCoefficient(sysParamList, peopleNum, averageGdp, U, I, II)));
         resultJsonObject.put("CE", df.format(electricConsumeStirCoefficient(sysParamList, CT, peopleNum2, averageGdp2, U2, PR, T)));

        return resultJsonObject;
    }

    public static List<Map> getThisModel(Integer args, List<SysModelParam> list) {
        List<Map> ary = new ArrayList<>();

        list.forEach((param) -> {
            if (param.getParamModel().equals(args)) {
                Map<String, Object> map = new HashMap<>();
                map.put("key", param.getParamName());
                map.put("value", param.getParamValue());
                ary.add(map);
            }
        });

        return ary;
    }

}
